'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = genDatabase;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _webpackConfig = require('./webpack.config.js');

var _webpackConfig2 = _interopRequireDefault(_webpackConfig);

var _letPluginsPostProcessData = require('./let-plugins-post-process-data');

var _letPluginsPostProcessData2 = _interopRequireDefault(_letPluginsPostProcessData);

var _eval = require('eval');

var _eval2 = _interopRequireDefault(_eval);

var _enablePlugins = require('./enable-plugins');

var _enablePlugins2 = _interopRequireDefault(_enablePlugins);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _memoryFs = require('memory-fs');

var _memoryFs2 = _interopRequireDefault(_memoryFs);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('graphql-directory-api:gen-database');
var fs = _fs2.default;

function genDatabase() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    watch: false,
    memoryFS: false,
    files: [],
    plugins: [],
    pluginOpts: {},
    outputDir: './dist'
  },
      _ref$watch = _ref.watch,
      watch = _ref$watch === undefined ? false : _ref$watch,
      _ref$memoryFS = _ref.memoryFS,
      memoryFS = _ref$memoryFS === undefined ? false : _ref$memoryFS,
      _ref$files = _ref.files,
      files = _ref$files === undefined ? [] : _ref$files,
      _ref$plugins = _ref.plugins,
      plugins = _ref$plugins === undefined ? [] : _ref$plugins,
      _ref$pluginOpts = _ref.pluginOpts,
      pluginOpts = _ref$pluginOpts === undefined ? {} : _ref$pluginOpts,
      _ref$outputDir = _ref.outputDir,
      outputDir = _ref$outputDir === undefined ? './dist' : _ref$outputDir;

  var callback = arguments[1];


  // Skip everything if there are no files to glob
  if (files && files.length === 0) {
    debug('No files in folder');
    return callback(null, []);
  }

  // use root directory as output if we're using memoryFS
  var distFolder = memoryFS ? '/' : (0, _path.resolve)(process.cwd(), outputDir);
  debug('distFolder', distFolder);
  debug('files', files);
  debug('dirs', process.cwd(), __dirname);
  var createdConfig = (0, _enablePlugins2.default)(plugins, pluginOpts, (0, _webpackConfig2.default)({
    filepaths: files,
    plugins: plugins
  }, {
    outputPath: distFolder
  }));

  var compiler = void 0;
  try {
    compiler = (0, _webpack2.default)(createdConfig);
  } catch (e) {
    console.log('webpack error');
    console.log(e.message);
    throw e;
  }

  if (memoryFS) {
    fs = new _memoryFs2.default();
    compiler.outputFileSystem = fs;
  } else {
    _mkdirp2.default.sync(distFolder);
  }

  function webpackHandler(err, stats) {
    // Start Error Checking
    if (err) {
      // hard failure
      debug('webpack failed', err);
      callback(err);
      return;
    }
    var jsonStats = stats.toJson();
    if (jsonStats.errors.length > 0) {
      //soft failure
      debug('webpack stats errors', jsonStats.errors[0]);
      callback(jsonStats.errors);
      return;
    }
    if (jsonStats.warnings.length > 0) {
      debug('webpack stats warnings', jsonStats.warnings);
      callback(jsonStats.warnings);
      return;
    }
    // End Error Checking
    debug('dist output', fs.readdirSync(distFolder));
    var apiJSONAsString = fs.readFileSync((0, _path.resolve)(distFolder, 'database-bundle.js'), 'utf-8');
    var apiJSON = (0, _eval2.default)(apiJSONAsString, 'api-database.json', null, true);
    debug('apiJSON count', apiJSON.length);
    (0, _letPluginsPostProcessData2.default)(plugins, apiJSON, function (err, resultingData) {
      if (err) {
        debug('caught err', err);
        // facilitate callback if error is thrown
        return callback(err);
      }
      debug('resultingData count', resultingData.length);
      return callback(null, resultingData);
    });
  }

  if (watch) {
    compiler.watch({
      aggregateTimeout: 300
    }, webpackHandler);
  } else {
    compiler.run(webpackHandler);
  }
}