'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = letPluginsPostProcessData;

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _waterfall = require('async/waterfall');

var _waterfall2 = _interopRequireDefault(_waterfall);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 
 * This file is `noParse` due to dynamic requires.
 *
 * Allow plugins access to the data loaded by webpack so that they can
 * generate, read, and delete content, etc.
 *
 * An example of this is removing blogposts which are `drafts` so they
 * aren't published yet or generating collections (including
 * pagination) for blog categories. (/blog/topic/1, /blog/topic/2)
 *
 */
var debug = (0, _debug2.default)('graphql-directory-api:let-plugins-post-process-data');
function letPluginsPostProcessData(plugins, data, callback) {
  var arr = [function (cb) {
    return cb(null, data);
  }];
  plugins.forEach(function (plugin) {
    try {
      // try to resolve plugin as node_module
      require.resolve(plugin + '/process');
      debug(plugin, 'wants to post-process the data');
      arr.push(require(plugin + '/process'));
    } catch (e) {
      try {
        var p = require.resolve(_path2.default.resolve(process.cwd(), plugin + '/process'));
        debug(plugin, 'is a local plugin that wants to post-process the data');
        arr.push(require(p));
      } catch (e) {
        debug('No process for plugin', plugin);
        return;
      }
    }
  });

  /**
   * allow each plugin to process the full data set
   * run order is not guarenteed.
   */
  debug('pre-waterfall count', data.length);
  (0, _waterfall2.default)(arr, function (err, result) {
    debug('post-waterfall count', result && result.length);
    if (err) {
      debug('a plugin threw an error when processing data');
      callback(err);
    } else {
      callback(null, result);
    }
  });
}